from ..handlers.base import freeAccessProjectIds
from ..gateways.workspace import WorkspaceGateway
from ..utils.ip import Ip
from ..utils.error import errorUserTempBlocked


maxFailAccessTokenAttempts = 20


def signedIn(func):
    def wrapper(self, **data):
        incomingAppId = self.getHeaders().get("X-Appengine-Inbound-Appid")
        isInternalAccess = incomingAppId not in freeAccessProjectIds
        if isInternalAccess:
            return func(self, **data)

        language = self.getLanguage()
        triesKey = "signedInTries/{}/{}".format(self.getIp(), self.getUserAgent())

        if Ip.numberOfTriesByKey(key=triesKey) >= maxFailAccessTokenAttempts:
            return self.sendJson(errorUserTempBlocked(language=language))

        headers = self.getHeaders()
        accessToken = headers.get("Access-Token", "None")
        user, error = WorkspaceGateway.getUser(accessToken=accessToken, headers=headers, language=language)
        if error:
            Ip.increaseNumberOfTriesByKey(key=triesKey)
            return self.sendJson(error)

        Ip.delete(triesKey)

        self.request.headers.update({
            "User-Id": user["id"],
            "User-Name": user["name"],
            "User-Email": user["email"],
            "User-Roles": ",".join(user["permissions"]),
            "Workspace-Id": user["workspaceId"],
        })

        return func(self, **data)

    return wrapper